

#include "IrrCompileConfig.h"



#ifdef _IRR_COMPILE_WITH_WINDOWS_DEVICE_


#define WIN32_LEAN_AND_MEAN
#if !defined(_IRR_XBOX_PLATFORM_)
	#include <windows.h>
	#include <mmsystem.h> // For JOYCAPS
	#include <windowsx.h>
#endif


#include "CJoystickWin32.h"


namespace irr
{




// some French doc available at http://tcharles.developpez.com
//bool
//CJoystickWin32::isConnected() {
//
//    JOYINFOEX joyInfo;
//    joyInfo.dwSize = sizeof(joyInfo);
//    joyInfo.dwFlags = 0;
//
//    //JOYSTICKID1 +
//    return joyGetPosEx( index, &joyInfo) == JOYERR_NOERROR;
//}
//


CJoystickWin32::CJoystickWin32(const char* name,const u8& JoyId,const SJoystickCapabilities& capabilities,JOYCAPS const& caps)  :
    IJoystick(name,JoyId,capabilities),wCaps(caps)
{

}






void
CJoystickWin32::update()
{
#if defined _IRR_COMPILE_WITH_JOYSTICK_EVENTS_
#ifdef _IRR_COMPILE_WITH_DIRECTINPUT_JOYSTICK_

#error not updated yet

#else // not _IRR_COMPILE_WITH_DIRECTINPUT_JOYSTICK_

	JOYINFOEX info; //!< Will store state

    // needs to be reset for each joystick
    // request ALL values and POV as continuous if possible
    info.dwSize = sizeof(info);
    info.dwFlags  = JOY_RETURNX | JOY_RETURNY | JOY_RETURNZ | JOY_RETURNR | JOY_RETURNU | JOY_RETURNV | JOY_RETURNBUTTONS;
    // return pov or not
    info.dwFlags |= (Capabilities.PovHat == POV_HAT_PRESENT ) ? JOY_RETURNPOVCTS : JOY_RETURNPOV;



    // retrive asked infos
    if(JOYERR_NOERROR == joyGetPosEx(Index, &info))
    {

        State.POV = (u16)info.dwPOV;


        // set to undefined if no POV value was returned or the value
        // is out of range
        if (!(info.dwFlags & JOY_RETURNPOV) || (State.POV > 35900)){
            State.POV = 65535;
        }

//SEvent::SJoystickEvent::NUMBER_OF_AXES
//			for(int axis = 0; axis < SJoystickCapabilites::MaximumNumberOfSupportedAxis; ++axis){
//				//event.JoystickEvent.Axis[axis] = 0;
//				State.Axis[]
//			}


        State.Buttons = info.dwButtons;

        // Set axis values
        switch(wCaps.wNumAxes)
        {
        default:
        case 6:
            State.Axis[SEvent::SJoystickEvent::AXIS_V] =
                (s16)((65535 * (info.dwVpos - wCaps.wVmin)) / (wCaps.wVmax - wCaps.wVmin) - 32768);

        case 5:
            State.Axis[SEvent::SJoystickEvent::AXIS_U] =
                (s16)((65535 * (info.dwUpos - wCaps.wUmin)) / (wCaps.wUmax - wCaps.wUmin) - 32768);

        case 4:
            State.Axis[SEvent::SJoystickEvent::AXIS_R] =
                (s16)((65535 * (info.dwRpos - wCaps.wRmin)) / (wCaps.wRmax - wCaps.wRmin) - 32768);

        case 3:
            State.Axis[SEvent::SJoystickEvent::AXIS_Z] =
                (s16)((65535 * (info.dwZpos - wCaps.wZmin)) / (wCaps.wZmax - wCaps.wZmin) - 32768);

        case 2:
            State.Axis[SEvent::SJoystickEvent::AXIS_Y] =
                (s16)((65535 * (info.dwYpos - wCaps.wYmin)) / (wCaps.wYmax - wCaps.wYmin) - 32768);

        case 1:
            State.Axis[SEvent::SJoystickEvent::AXIS_X] =
                (s16)((65535 * (info.dwXpos - wCaps.wXmin)) / (wCaps.wXmax - wCaps.wXmin) - 32768);
        }

        // post events
        //(void)Device->postEventFromUser(event);
    }

    // ! JOYERR_NOERROR
    else {

        // TODO

    }

#endif
#endif // _IRR_COMPILE_WITH_JOYSTICK_EVENTS_
}




/*
//////////////////////////////////////////////////////
void
IJoystick::update()
{
    //JoystickState state;

    // Get the current joystick state
    JOYINFOEX pos;
    pos.dwFlags  = JOY_RETURNX | JOY_RETURNY | JOY_RETURNZ | JOY_RETURNR | JOY_RETURNU | JOY_RETURNV | JOY_RETURNBUTTONS;
    pos.dwFlags |= (myCaps.wCaps & JOYCAPS_POVCTS) ? JOY_RETURNPOVCTS : JOY_RETURNPOV;
    pos.dwSize   = sizeof(JOYINFOEX);
    if (joyGetPosEx(myIndex, &pos) == JOYERR_NOERROR)
    {
        // The joystick is connected
        state.Connected = true;

        // Axes
        state.Axes[Joystick::X] = (pos.dwXpos - (myCaps.wXmax + myCaps.wXmin) / 2.f) * 200.f / (myCaps.wXmax - myCaps.wXmin);
        state.Axes[Joystick::Y] = (pos.dwYpos - (myCaps.wYmax + myCaps.wYmin) / 2.f) * 200.f / (myCaps.wYmax - myCaps.wYmin);
        state.Axes[Joystick::Z] = (pos.dwZpos - (myCaps.wZmax + myCaps.wZmin) / 2.f) * 200.f / (myCaps.wZmax - myCaps.wZmin);
        state.Axes[Joystick::R] = (pos.dwRpos - (myCaps.wRmax + myCaps.wRmin) / 2.f) * 200.f / (myCaps.wRmax - myCaps.wRmin);
        state.Axes[Joystick::U] = (pos.dwUpos - (myCaps.wUmax + myCaps.wUmin) / 2.f) * 200.f / (myCaps.wUmax - myCaps.wUmin);
        state.Axes[Joystick::V] = (pos.dwVpos - (myCaps.wVmax + myCaps.wVmin) / 2.f) * 200.f / (myCaps.wVmax - myCaps.wVmin);

        // Special case for POV, it is given as an angle
        if (pos.dwPOV != 0xFFFF)
        {
            float angle = pos.dwPOV / 36000.f * 3.141592654f;
            state.Axes[Joystick::PovX] = std::cos(angle) * 100;
            state.Axes[Joystick::PovY] = std::sin(angle) * 100;
        }
        else
        {
            state.Axes[Joystick::PovX] = 0;
            state.Axes[Joystick::PovY] = 0;
        }

        // Buttons
        for (unsigned int i = 0; i < Joystick::ButtonCount; ++i)
            state.Buttons[i] = (pos.dwButtons & (1 << i)) != 0;
    }

    return state;
}
*/


}  // namespace irr



#endif // _IRR_COMPILE_WITH_WINDOWS_DEVICE_
